<?php

namespace Drupal\custom_configurations;

use Drupal\Core\Config\ConfigFactory;
use Drupal\Core\Extension\ModuleHandlerInterface;
use Drupal\Core\KeyValueStore\KeyValueFactory;
use Drupal\Core\Language\Language;
use Drupal\Core\Language\LanguageInterface;
use Drupal\Core\Language\LanguageManagerInterface;
use Drupal\Core\Session\AccountProxy;

/**
 * Helper for locale settings/variable-related methods.
 *
 * @package Drupal\custom_configurations
 */
class CustomConfigurationsManager {

  /**
   * The config factory service.
   *
   * @var \Drupal\Core\Config\ConfigFactory
   */
  protected $configFactory;

  /**
   * The language manager.
   *
   * @var \Drupal\Core\Language\LanguageManagerInterface
   */
  protected $languageManager;

  /**
   * The language manager.
   *
   * @var \Drupal\Core\Extension\ModuleHandlerInterface
   */
  protected $moduleHandler;

  /**
   * Default key/value store service.
   *
   * @var \Drupal\Core\KeyValueStore\KeyValueFactory
   */
  protected $keyValue;

  /**
   * The custom configurations plugin manager.
   *
   * @var \Drupal\custom_configurations\CustomConfigurationsPluginManager
   */
  protected $pluginManager;

  /**
   * The current user.
   *
   * @var \Drupal\Core\Session\AccountProxy
   */
  protected $currentUser;

  /**
   * Constructor.
   *
   * @param \Drupal\Core\Config\ConfigFactory $config_factory
   *   The factory for configuration objects.
   * @param \Drupal\Core\Language\LanguageManagerInterface $language_manager
   *   Language manager service.
   * @param \Drupal\Core\Extension\ModuleHandlerInterface $module_handler
   *   The ModuleHandlerInterface.
   * @param \Drupal\Core\KeyValueStore\KeyValueFactory $keyvalue
   *   Default key/value store service.
   * @param \Drupal\custom_configurations\CustomConfigurationsPluginManager $plugin_manager
   *   The custom configurations plugin manager.
   * @param \Drupal\Core\Session\AccountProxy $current_user
   *   The current user.
   */
  public function __construct(ConfigFactory $config_factory, LanguageManagerInterface $language_manager, ModuleHandlerInterface $module_handler, KeyValueFactory $keyvalue, CustomConfigurationsPluginManager $plugin_manager, AccountProxy $current_user) {
    $this->configFactory = $config_factory;
    $this->languageManager = $language_manager;
    $this->moduleHandler = $module_handler;
    $this->keyValue = $keyvalue;
    $this->pluginManager = $plugin_manager;
    $this->currentUser = $current_user;
  }

  /**
   * Get a localized variable from the configs.
   *
   * @param string $plugin_id
   *   Which config to fetch a value from?
   * @param string $var_name
   *   The id of the value to fetch.
   * @param string|object|null|false $language
   *   The language to get the config from.
   *
   * @return mixed|array|null
   *   May return a value for a given $var_name,
   *   array of all values for the current $plugin_id
   *   or NULL if nothing is found.
   */
  public function getFileConfig($plugin_id, $var_name = NULL, $language = NULL) {
    $variable = NULL;
    $original_language = $this->languageManager->getConfigOverrideLanguage();
    // Priority #1: Language-specific variable.
    if ($language && $this->languagesAvailable()) {
      $language = $this->getApplicableLanguageObject($language);
      $this->languageManager->setConfigOverrideLanguage($language);
      $key = $this->getConfigKey($plugin_id, $language->getId());
      $config = $this->configFactory->getEditable($key);
      if ($config && $var_name === NULL) {
        return $config;
      }
      if ($config) {
        $variable = $config->get($var_name);
      }
    }
    // Priority #2: Default variable.
    if (!$variable) {
      $this->languageManager->setConfigOverrideLanguage($original_language);
      $key = $this->getConfigKey($plugin_id);
      $config = $this->configFactory->getEditable($key);
      if ($config && $var_name === NULL) {
        return $config;
      }
      if ($config) {
        $variable = $config->get($var_name);
      }
    }

    return $variable;
  }

  /**
   * Get a localized variable from the database configs.
   *
   * @param string $plugin_id
   *   Which config to fetch a value from?
   * @param string $var_name
   *   The id of the value to fetch.
   * @param string|object|null|false $language
   *   The language to get the config from.
   *
   * @return mixed|array|null
   *   May return a value for a given $var_name,
   *   array of all values for the current $plugin_id
   *   or NULL if nothing is found.
   */
  public function getDbConfig($plugin_id, $var_name = NULL, $language = NULL) {
    $variable = NULL;
    // Priority #1: Language-specific variable.
    if ($language && $this->languagesAvailable()) {
      $language = $this->getApplicableLanguageObject($language);
      $key = $this->getConfigKey($plugin_id, $language->getId());
      $config = $this->keyValue->get($key);
      if ($config && $var_name === NULL) {
        return $config;
      }
      if ($config) {
        $variable = $config->get($var_name);
      }
    }
    // Priority #2: Default variable.
    if (!$variable) {
      $key = $this->getConfigKey($plugin_id);
      $config = $this->keyValue->get($key);
      if ($config && $var_name === NULL) {
        return $config;
      }
      if ($config) {
        $variable = $config->get($var_name);
      }
    }
    return $variable;
  }

  /**
   * Generates key for data storage.
   *
   * @param string $plugin_id
   *   Which config to fetch a value from?
   * @param string|object|null $language
   *   The language to get the config from.
   *
   * @return string
   */
  public function getConfigKey($plugin_id, $language = NULL) {
    $key = 'custom_configurations.';
    $key .= $language ? $plugin_id . '.' . $language : $plugin_id;
    return $key;
  }

  /**
   * Check if module language enabled.
   *
   * @return bool
   *   Returns true or false.
   */
  public function languagesAvailable() {
    return $this->moduleHandler->moduleExists('language');
  }

  /**
   * Takes a langcode or language object and returns an object.
   *
   * @param string|LanguageInterface|null $language
   *   The language to be tested.
   *
   * @return LanguageInterface
   */
  public function getApplicableLanguageObject($language) {
    if ($language instanceof Language) {
      return $language;
    }
    if ($language) {
      if (is_string($language)) {
        $language_object = $this->languageManager->getLanguage($language);
        if ($language_object) {
          return $language_object;
        }
      }
    }
    return $this->languageManager->getCurrentLanguage();
  }

  /**
   * Retrieves configuration plugins.
   *
   * @return array
   */
  public function getConfigPlugins() {
    $plugins = [];
    // Fetch the user's roles to check against plugin definition permissions.
    $acct_roles = $this->currentUser->getRoles();

    // Fetch the registered config plugins from all modules...
    $plugin_definitions = $this->pluginManager->getDefinitions();

    // ...sort them by weight...
    uasort($plugin_definitions, function ($a, $b) {
      if ($a['weight'] == $b['weight']) {
        return 0;
      }

      return ($a['weight'] < $b['weight']) ? -1 : 1;
    });
    // ...and iterate through them.
    foreach ($plugin_definitions as $definition) {
      // Don't show modules hard-coded as "disabled".
      if (empty($definition['disabled'])) {
        // Only check roles IF plugin has allowed roles defined.
        if (!empty($definition['allowed_roles'])) {
          // Ensure the current user is permitted to see the config.
          if (!array_intersect($acct_roles, $definition['allowed_roles'])) {
            continue;
          }
        }

        // Fetch the plugin's form elements.
        $plugins[$definition['id']] = $definition;
      }
    }

    return $plugins;
  }

}
